@@ -5,6 +5,7 @@ from django import VERSION as django_version
 from django.core import exceptions
 from django.core import serializers
 from django.db import models as django_models
+from django import test
 from django.utils import unittest
 
 import xworkflows
@@ -22,7 +23,7 @@ except ImportError:
 from . import models
 
 
-class ModelTestCase(unittest.TestCase):
+class ModelTestCase(test.TestCase):
     def test_workflow(self):
         self.assertEqual(models.MyWorkflow.states,
                          models.MyWorkflowEnabled._workflows['state'].workflow.states)
@@ -136,7 +137,7 @@ class ModelTestCase(unittest.TestCase):
             serializers.deserialize('json', data))
 
 
-class InheritanceTestCase(unittest.TestCase):
+class InheritanceTestCase(test.TestCase):
     """Tests inheritance-related behaviour."""
     def test_simple(self):
         class BaseWorkflowEnabled(xwf_models.WorkflowEnabled, django_models.Model):
@@ -161,7 +162,8 @@ class InheritanceTestCase(unittest.TestCase):
         self.assertEqual(models.MyWorkflow.initial_state, obj.state)
 
 
-class TransitionTestCase(unittest.TestCase):
+# Not a standard TestCase, since we're testing transactions.
+class TransitionTestCase(test.TransactionTestCase):
 
     def setUp(self):
         self.obj = models.MyWorkflowEnabled()
@@ -250,8 +252,70 @@ class TransitionTestCase(unittest.TestCase):
         self.assertEqual(models.MyWorkflow.states.foo, obj.state)
 
 
+class LastTransitionLogTestCase(test.TestCase):
+    def setUp(self):
+        self.obj = models.SomeWorkflowEnabled.objects.create()
+
+    def test_transitions(self):
+        self.assertEqual(0, models.SomeWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ab()
+        self.assertEqual(1, models.SomeWorkflowLastTransitionLog.objects.count())
+        tlog = models.SomeWorkflowLastTransitionLog.objects.get()
+        self.assertEqual(self.obj, tlog.obj)
+        self.assertEqual('ab', tlog.transition)
+        self.assertEqual('a', tlog.from_state)
+        self.assertEqual('b', tlog.to_state)
+
+    def test_two_transitions(self):
+        self.assertEqual(0, models.SomeWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ab()
+        self.assertEqual(1, models.SomeWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ba()
+        self.assertEqual(1, models.SomeWorkflowLastTransitionLog.objects.count())
+
+        tlog = models.SomeWorkflowLastTransitionLog.objects.get()
+        self.assertEqual(self.obj, tlog.obj)
+        self.assertEqual('ba', tlog.transition)
+        self.assertEqual('b', tlog.from_state)
+        self.assertEqual('a', tlog.to_state)
+
+
+class GenericLastTransitionLogTestCase(test.TestCase):
+    def setUp(self):
+        self.obj = models.GenericWorkflowEnabled.objects.create()
+
+    def test_transitions(self):
+        self.assertEqual(0, models.GenericWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ab()
+        self.assertEqual(1, models.GenericWorkflowLastTransitionLog.objects.count())
+        tlog = models.GenericWorkflowLastTransitionLog.objects.get()
+        self.assertEqual(self.obj, tlog.modified_object)
+        self.assertEqual('ab', tlog.transition)
+        self.assertEqual('a', tlog.from_state)
+        self.assertEqual('b', tlog.to_state)
+
+    def test_two_transitions(self):
+        self.assertEqual(0, models.GenericWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ab()
+        self.assertEqual(1, models.GenericWorkflowLastTransitionLog.objects.count())
+
+        self.obj.ba()
+        self.assertEqual(1, models.GenericWorkflowLastTransitionLog.objects.count())
+
+        tlog = models.GenericWorkflowLastTransitionLog.objects.get()
+        self.assertEqual(self.obj, tlog.modified_object)
+        self.assertEqual('ba', tlog.transition)
+        self.assertEqual('b', tlog.from_state)
+        self.assertEqual('a', tlog.to_state)
+
+
 @unittest.skipIf(south is None, "Couldn't import south.")
-class SouthTestCase(unittest.TestCase):
+class SouthTestCase(test.TestCase):
     """Tests south-related behavior."""
 
     frozen_workflow = (
